package cmd

import (
	"context"
	"fmt"
	"log/slog"
	"os"
	"os/signal"
	"syscall"

	"github.com/spf13/cobra"
	"gitlab.com/gitlab-org/cluster-integration/gitlab-agent/v17/internal/tool/logz"
)

func Run(command *cobra.Command) {
	command.Version = fmt.Sprintf("%s, git ref: %s", Version, GitRef)
	err := run(command)
	if err != nil {
		// Error is logged in the command itself using a proper logger.
		os.Exit(1)
	}
}

func run(command *cobra.Command) error {
	ctx, cancelFunc := signal.NotifyContext(context.Background(), os.Interrupt, syscall.SIGTERM)
	defer cancelFunc()

	return command.ExecuteContext(ctx)
}

func RunAndLog(ctx context.Context, run func(context.Context) (*slog.Logger, error)) error {
	log, err := run(ctx)
	if err != nil {
		if log == nil {
			log = slog.New(slog.NewJSONHandler(os.Stderr, nil))
		}
		log.Error("Program aborted", logz.Error(err))
		return err
	}
	return nil
}
